-- 	Placeable FuelTrigger and AnimatedObject Script
--
-- 	Author: 	Giants Software, Silas770
-- 	Date:		22.1.2017
--		Version:	1.0 (LS17)
--
--		Contact at NKB Modding:
--		https://nkb-modding.com/index.php/User/6877-Silas770/
-- 	Contact at Vertex Dezign:
--		http://vertexdezign.net/forum/index.php/User/2081-Silas770/
--

PlaceableFTAO = {};

PlaceableFTAO_mt = Class(PlaceableFTAO, Placeable);

InitObjectClass(PlaceableFTAO, "PlaceableFTAO");

function PlaceableFTAO:new(isServer, isClient, customMt)
    local mt = customMt;
    if mt == nil then
        mt = PlaceableFTAO_mt;
    end;
	
    local self = Placeable:new(isServer, isClient, mt);
	
    registerObjectClassName(self, "PlaceableFTAO");
	
    return self;
end;

function PlaceableFTAO:delete()
    if self.animatedObjects ~= nil then
        for i=1, #self.animatedObjects do
            local animatedObject = self.animatedObjects[i]
            animatedObject:unregister(true)
            animatedObject:delete()
        end
    end
	
	if self.fillTrigger ~= nil then
        self.fillTrigger:delete();
    end;
	
    unregisterObjectClassName(self);
    PlaceableFTAO:superClass().delete(self);
end;

function PlaceableFTAO:load(xmlFilename, x,y,z, rx,ry,rz, initRandom)
    if not PlaceableFTAO:superClass().load(self, xmlFilename, x,y,z, rx,ry,rz, initRandom) then
        return false;
    end;
	
    local xmlFile = loadXMLFile("TempXML", xmlFilename);
	
    self.animatedObjectNodes = {}
    self.animatedObjects = {}
	
    local i = 0
    while true do
        local key = string.format("placeable.animatedObjects.animatedObject(%d)", i)
        if not hasXMLProperty(xmlFile, key) then
            break
        end
		
        local node = Utils.indexToObject(self.nodeId, getXMLString(xmlFile, key.."#index"))
        if node ~= nil then
            table.insert(self.animatedObjectNodes, node)
        end
        i = i + 1
    end
	
	self.fillTriggerId = Utils.indexToObject(self.nodeId, getXMLString(xmlFile, "placeable.fillTrigger#index"))
	
	self.fillTrigger = GasStation:new(self.fillTriggerId);
    if self.fillTrigger ~= nil then
        g_currentMission:addNonUpdateable(self.fillTrigger);
	end;
	
    delete(xmlFile);
	
    return true;
end;

function PlaceableFTAO:finalizePlacement()
    PlaceableFTAO:superClass().finalizePlacement(self);
	
    for _, node in pairs(self.animatedObjectNodes) do
        local animatedObject = AnimatedObject:new(self.isServer, self.isClient)
        animatedObject.baseDirectory = self.baseDirectory
        animatedObject.customEnvironment = self.customEnvironment
        if animatedObject:load(node) then
            animatedObject:register(true)
            table.insert(self.animatedObjects, animatedObject)
        end
    end
end

function PlaceableFTAO:readStream(streamId, connection)
    PlaceableFTAO:superClass().readStream(self, streamId, connection);
    if connection:getIsServer() then
        for i=1, #self.animatedObjects do
            local animatedObject = self.animatedObjects[i]
            local animatedObjectId = readNetworkNodeObjectId(streamId);
            animatedObject:readStream(streamId, connection);
            g_client:finishRegisterObject(animatedObject, animatedObjectId);
        end
    end
end

function PlaceableFTAO:writeStream(streamId, connection)
    PlaceableFTAO:superClass().writeStream(self, streamId, connection);
    if not connection:getIsServer() then
        for i=1, #self.animatedObjects do
            local animatedObject = self.animatedObjects[i]
            writeNetworkNodeObjectId(streamId, animatedObject.id)
            animatedObject:writeStream(streamId, connection)
            g_server:registerObjectInStream(connection, animatedObject)
        end
    end
end

function PlaceableFTAO:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
    if not PlaceableFTAO:superClass().loadFromAttributesAndNodes(self, xmlFile, key, resetVehicles) then
        return false;
    end;
	
    for i=1, #self.animatedObjects do
        local animatedObject = self.animatedObjects[i]
        if hasXMLProperty(xmlFile, key..".animatedObject("..(i-1)..")") then
            animatedObject:loadFromAttributesAndNodes(xmlFile, key..".animatedObject("..(i-1)..")")
        end
    end
	
    return true;
end;

function PlaceableFTAO:getSaveAttributesAndNodes(nodeIdent)
    local attributes, nodes = PlaceableFTAO:superClass().getSaveAttributesAndNodes(self, nodeIdent);
	
    for i=1, #self.animatedObjects do
        local animatedObject = self.animatedObjects[i]
        local a, _ = animatedObject:getSaveAttributesAndNodes(nodeIdent .. "    ")
        if i > 1 then
            nodes = nodes.."\n";
        end;
        nodes = nodes..nodeIdent.."<animatedObject".." "..a.. "/>"
    end
	
    return attributes, nodes;
end;

registerPlaceableType("placeableFTAO", PlaceableFTAO);
